@@ -4,7 +4,6 @@
  * See COPYING in top-level directory.
  */
 
-#define _GNU_SOURCE
 #include <unistd.h>
 #include <sys/types.h>
 #include <string.h>
@@ -16,8 +15,15 @@
 #include <signal.h>
 #include <execinfo.h>
 #include <sys/syscall.h>
+
 #include "ev_codes.h"
 #include "eztrace.h"
+#include "eztrace_config.h"
+
+#if HAVE_LIBBACKTRACE
+#include <libbacktrace/backtrace.h>
+#include <libbacktrace/backtrace-supported.h>
+#endif
 
 static void __eztrace_run_atexit_functions();
 
@@ -25,6 +31,7 @@ static pthread_key_t protect_on;
 
 struct __ezt_write_trace __ezt_trace;
 
+ 
 static void init_recursion_shield() {
   static int init_done = 0;
   if (!init_done) {
@@ -51,7 +58,7 @@ static void __eztrace_set_buffer_size() {
 static char* __eztrace_get_filedir() {
   char* res = getenv("EZTRACE_TRACE_DIR");
   if (!res)
-    asprintf(&res, "/tmp");
+    asprintf(&res, ".");
   return res;
 }
 
@@ -104,8 +111,8 @@ static char* get_lib_name(char*symbol) {
 static char* get_function_name(char* symbol) {
   char* ret = symbol;
   int len = strlen(symbol);
-  int begin;
-  int end;
+  int begin=0;
+  int end=0;
   int i;
 
   /* the format of symbol is : 'libxxx.so (function+0xoffset) [0xaddress]*/
@@ -122,12 +129,15 @@ static char* get_function_name(char* symbol) {
       break;
     }
   }
-
+  if(begin == 0) {
+    return symbol;
+  }
   ret = &symbol[begin + 1];
   symbol[end] = 0;
   return ret;
 }
 
+
 void eztrace_error_handler(int signo) {
   static volatile int shield = 0;
   /* in case several thread receive signals simultaneously, only the first one
@@ -139,22 +149,18 @@ void eztrace_error_handler(int signo) {
 
   set_recursion_shield_on();
 
-  EZT_PRINTF(0, "[EZTrace] signal %d catched.\n", signo);
-  void* buffer[50];
+  EZT_PRINTF(0, "[EZTrace] signal %d catched. my pid=%d\n", signo, getpid());
 
+  void* buffer[50];
   /* get pointers to functions */
   int nb_calls = backtrace(buffer, 50);
   char **functions;
 
   functions = backtrace_symbols(buffer, nb_calls);
   int i;
-  EZTRACE_EVENT2(EZTRACE_SIGNAL_RECEIVED, signo, nb_calls);
-  for (i = 0; i < nb_calls; i++) {
-    char* ptr = get_function_name(functions[i]);
-    litl_write_probe_raw(__ezt_trace.litl_trace, EZTRACE_CALLING_FUNCTION, strlen(ptr), (void*)ptr);
-  }
-  free(functions);
 
+  EZTRACE_EVENT_PACKED_2(EZTRACE_SIGNAL_RECEIVED, signo, nb_calls);
+  eztrace_record_backtrace(nb_calls);
   set_recursion_shield_off();
 }
 
@@ -197,7 +203,7 @@ void eztrace_set_alarm() {
     sevp.sigev_value.sival_int=0;
     sevp.sigev_notify_function = NULL;
     sevp.sigev_notify_attributes=NULL;
-    sevp._sigev_un._tid = syscall(__NR_gettid);
+    sevp._sigev_un._tid = CUR_TID;
 
     timer_t *t = malloc(sizeof(timer_t));
     int ret = timer_create(CLOCK_REALTIME, &sevp, t);
@@ -276,6 +282,7 @@ void eztrace_start() {
   atexit(eztrace_stop);
   __eztrace_set_sighandler();
 
+
   char* allow_flush = getenv("EZTRACE_FLUSH");
   // start LiTL
   __ezt_trace.litl_trace = litl_write_init_trace(__ezt_trace.buffer_size);
@@ -486,18 +493,89 @@ int recursion_shield_on() {
 
 void set_recursion_shield_on() {
   init_recursion_shield();
-  pthread_setspecific(protect_on, (void*) 1);
+   pthread_setspecific(protect_on, (void*) 1);
 }
 void set_recursion_shield_off() {
   pthread_setspecific(protect_on, (void*) NULL);
 }
 
-#ifdef RECORD_BACKTRACES
-/* record an event (code=EZTRACE_CALLING_FUNCTION) with the calling function name */
-void record_backtrace()
-{
-  set_recursion_shield_on();
+static void  __eztrace_get_function_name_from_pointer_fallback(void* pointer,
+							       char *output_str,
+							       int buffer_len) {
+  char **functions;
+  functions = backtrace_symbols(pointer, 1);
+  char* ptr = get_function_name(functions[0]);
+  strncpy(output_str, ptr, strlen(ptr)+1);
+  free(functions);
+}
 
+#if HAVE_LIBBACKTRACE
+
+__thread struct backtrace_state *backtrace_state = NULL;
+__thread char current_frame[1024];
+
+static void error_callback(void *data, const char *msg, int errnum) {
+  fprintf(stderr, "ERROR: %s (%d)", msg, errnum);
+}
+
+static int backtrace_callback (void *data, uintptr_t pc,
+			       const char *filename, int lineno,
+			       const char *function) {
+  if(filename == NULL) {
+    /* cannot find debugging info */
+    __eztrace_get_function_name_from_pointer_fallback((void*)pc, current_frame, 1024);
+  } else {
+    snprintf(current_frame, 1024, "at %s:%d %s", filename, lineno, function);
+  }
+  return 0;
+}
+#endif /* HAVE_LIBBACKTRACE */
+
+
+/* retrieve the name of the function corresponding to pointer
+ * the collected information is copied into output_str
+ * buffer_len is the maximum number of bytes written to output_str
+ */
+static void __eztrace_get_function_name_from_pointer(void* pointer,
+						     char *output_str,
+						     int buffer_len) {
+#if HAVE_LIBBACKTRACE
+  if(!backtrace_state) {
+    backtrace_state = backtrace_create_state (NULL, 0, error_callback, NULL);
+  }
+  backtrace_pcinfo (backtrace_state, (uintptr_t)pointer,
+		    backtrace_callback,
+		    error_callback,
+		    NULL);
+
+  strncpy(output_str, current_frame, strlen(current_frame)+1);
+#else
+  __eztrace_get_function_name_from_pointer_fallback(pointer, output_str, buffer_len);
+#endif
+
+}
+
+/* get information on frame frameid
+ * the collected information is copied into output_str
+ * buffer_len is the maximum number of bytes written to output_str
+ */
+void eztrace_get_stack_frame(int frameid,
+			     char*output_str,
+			     int buffer_len) {
+  void* buffer[frameid+1];
+
+  /* get pointers to functions */
+  int nb_calls = backtrace(buffer, frameid+1);
+  __eztrace_get_function_name_from_pointer(buffer[frameid],
+					   output_str,
+					   buffer_len);
+}
+
+#define RECORD_BACKTRACES 1
+#ifdef RECORD_BACKTRACES
+
+/* record events (code=EZTRACE_CALLING_FUNCTION) containing the backtrace */
+void eztrace_record_backtrace(int backtrace_depth) {
   /* the current backtrace looks like this:
    * 0 - record_backtrace()
    * 1 - eztrace_callback()
@@ -505,20 +583,31 @@ void record_backtrace()
    *
    * So, we need to get the name of the function in frame 2.
    */
-  void* buffer[3];
+  void* bt_buffer[backtrace_depth];
+
   /* get pointers to functions */
+  int nb_calls = backtrace(bt_buffer, backtrace_depth);
 
-  int nb_calls = backtrace(buffer, 3);
-  assert(nb_calls>=3);
+  EZTRACE_EVENT1_PACKED_UNPROTECTED(EZTRACE_BACKTRACE_DEPTH, nb_calls);
+  int i;
+  for (i = 0; i < nb_calls; i++) {
+    char buffer[1024];
+    __eztrace_get_function_name_from_pointer(bt_buffer[i], buffer, 1024);
+    litl_write_probe_raw(__ezt_trace.litl_trace, EZTRACE_CALLING_FUNCTION, strlen(buffer), buffer);
+  }
+}
 
-  char **functions;
-  functions = backtrace_symbols(buffer, nb_calls);
-  litl_write_probe_raw(__ezt_trace.litl_trace, EZTRACE_CALLING_FUNCTION, strlen(functions[2]), functions[2]);
-  free(functions);
-  set_recursion_shield_off();
+void record_backtrace() {
+  EZTRACE_PROTECT {
+    EZTRACE_PROTECT_ON();
+    eztrace_record_backtrace(15);
+    EZTRACE_PROTECT_OFF();
+  }
 }
 
 #else
+void eztrace_record_backtrace(int backtrace_depth) {
+}
 void record_backtrace() {
 }
 #endif
