@@ -22,7 +22,7 @@ struct hierarchical_array* hierarchical_array_find(
   if (p_cont)
     arrays = p_cont->arrays;
 
-  int i;
+  unsigned int i;
   for (i = 0; i < nb_arrays; i++) {
     if (arrays[i].id == counter_id)
       return &arrays[i];
@@ -31,7 +31,7 @@ struct hierarchical_array* hierarchical_array_find(
 }
 
 static void __hierarchical_array_expand(struct hierarchical_array* p_array,
-                                        unsigned nb_item_to_alloc) {
+					unsigned nb_item_to_alloc) {
   p_array->nb_allocated += nb_item_to_alloc;
   /* initialize the new array */
   struct hierarchical_array_data* new_array = malloc(
@@ -54,7 +54,7 @@ static void __hierarchical_array_expand(struct hierarchical_array* p_array,
 }
 
 static void __hierarchical_array_init(struct hierarchical_array* p_array,
-                                      eztrace_counter_id id, unsigned item_size) {
+				      eztrace_counter_id id, unsigned item_size) {
   assert(p_array);
   p_array->id = id;
   p_array->item_size = item_size;
@@ -69,7 +69,7 @@ static void __hierarchical_array_init(struct hierarchical_array* p_array,
  * vector_size items. The names of each of the vector item are stores in names
  */
 void hierarchical_array_attach(eztrace_counter_id counter_id,
-                               unsigned item_size) {
+			       unsigned item_size) {
   if (nb_arrays >= MAX_ARRAYS) {
     fprintf(stderr, "Too many arrays\n");
     exit(1);
@@ -92,27 +92,27 @@ void hierarchical_array_new_container(p_eztrace_container p_cont) {
   assert(p_cont);
 
   p_cont->arrays = malloc(sizeof(struct hierarchical_array) * nb_arrays);
-  int i;
+  unsigned int i;
   for (i = 0; i < nb_arrays; i++) {
     __hierarchical_array_init(&(p_cont->arrays[i]), root_arrays[i].id,
-                              root_arrays[i].item_size);
+			      root_arrays[i].item_size);
   }
 }
 
 /* return the number of items in the array (for a specific container) */
 unsigned hierarchical_array_size(p_eztrace_container p_cont,
-                                 eztrace_counter_id counter_id) {
+				 eztrace_counter_id counter_id) {
   struct hierarchical_array* array = hierarchical_array_find(counter_id,
-                                                             p_cont);
+							     p_cont);
   assert(array);
   return array->nb_items;
 }
 
 /* return the ith items of an array (for a specific container) */
 void* hierarchical_array_get_item(p_eztrace_container p_cont,
-                                  eztrace_counter_id counter_id, unsigned index) {
+				  eztrace_counter_id counter_id, unsigned index) {
   struct hierarchical_array* array = hierarchical_array_find(counter_id,
-                                                             p_cont);
+							     p_cont);
   assert(array);
   assert(array->nb_items > index);
   return ITH_ITEM(index, array);
@@ -120,9 +120,9 @@ void* hierarchical_array_get_item(p_eztrace_container p_cont,
 
 /* return a new item of an array (for a specific container) */
 void* hierarchical_array_new_item(p_eztrace_container p_cont,
-                                  eztrace_counter_id counter_id) {
+				  eztrace_counter_id counter_id) {
   struct hierarchical_array* array = hierarchical_array_find(counter_id,
-                                                             p_cont);
+							     p_cont);
   assert(array);
 
   if (array->nb_items >= array->nb_allocated) {
@@ -135,4 +135,3 @@ void* hierarchical_array_new_item(p_eztrace_container p_cont,
   array->nb_items++;
   return res;
 }
-
