@@ -4,7 +4,6 @@
  * See COPYING in top-level directory.
  */
 
-#define _GNU_SOURCE         /* See feature_test_macros(7) */
 #include <stdio.h>
 #include <assert.h>
 #include "mpi_ev_codes.h"
@@ -26,6 +25,7 @@ enum pending_comm_status_t {
   NONE_DONE = 1, IRECV_DONE = 2, ISEND_DONE = 4, RWAIT_DONE = 8, SWAIT_DONE = 16,
 };
 
+
 struct mpi_pending_comm {
   struct ezt_list_token_t token;
   struct mpi_p2p_msg_t *msg;
@@ -159,7 +159,7 @@ struct mpi_p2p_msg_t* __new_p2p_message(
   msg->unexp = unexp; /* set to 1 if the recv is detected before the send */
   int i;
   for (i = 0; i < P2P_NB_TIMES; i++)
-    msg->times[i] = 0;
+    msg->times[i] = TIME_INIT;
   msg->link_value = link_value;
   msg->sender_thread_id = sender_thread_id;
   msg->recver_thread_id = recver_thread_id;
@@ -274,9 +274,9 @@ __mpi_find_p2p_message(int src __attribute__((unused)), int dest,
     msg = ITH_ITEM(index, array);
 
     if (((msg->src == src) || (msg->src == p_info->__MPI_ANY_SOURCE))
-      && (msg->dest == dest)
-      && ((msg->tag == tag) || (msg->tag == p_info->__MPI_ANY_TAG))
-      && (time_id < 0 || !msg->times[time_id])) {
+	&& (msg->dest == dest)
+	&& ((msg->tag == tag) || (msg->tag == p_info->__MPI_ANY_TAG))
+	&& (time_id < 0 || !IS_TIME_SET(msg->times[time_id]))) {
       return msg;
     }
   }
@@ -294,7 +294,12 @@ __mpi_p2p_find_pending_comm(int src, int dest, int len __attribute__((unused)),
   struct mpi_process_info_t * process_info = NULL;
 
   INIT_MPI_PROCESS_INFO(GET_PROCESS_INFO(dest), p_info);
-
+#if DEBUG_MSG_MATCHING
+  printf("Searching for an MPI comm that matches (src=%x, dest=%x, tag=%x, type=%x, mask_ok=%x, mask_not_ok=%x)\n", src, dest, tag, type, mask_ok, mask_not_ok);
+  if(src==p_info->__MPI_ANY_SOURCE) {
+    printf("\t(src= MPI_ANY_SOURCE)\n");
+  }
+#endif
   switch (type) {
   case comm_type_outgoing: {
     INIT_MPI_PROCESS_INFO(GET_PROCESS_INFO(src), p_info_src);
@@ -317,16 +322,34 @@ __mpi_p2p_find_pending_comm(int src, int dest, int len __attribute__((unused)),
     struct mpi_pending_comm* p_comm = (struct mpi_pending_comm*) token->data;
     assert(p_comm->msg);
     msg = p_comm->msg;
-
+#if DEBUG_MSG_MATCHING
+    printf("\tcomm %p / msg %p (src=%x, dest=%x, tag=%x, status=%d)\n", p_comm, msg, msg->src, msg->dest, msg->tag, p_comm->status);
+#endif
     if ((msg->dest == dest)
-      && ((msg->src == src) || (msg->src == p_info->__MPI_ANY_SOURCE)
-	|| (src == p_info->__MPI_ANY_SOURCE))
-      && ((msg->tag == tag) || (msg->tag == p_info->__MPI_ANY_TAG)
-	|| (tag == p_info->__MPI_ANY_TAG)) && (p_comm->status & mask_ok)
-      && (!(p_comm->status & mask_not_ok))) {
-      return p_comm;
+	&& ((msg->src == src) || (msg->src == p_info->__MPI_ANY_SOURCE) || (src == p_info->__MPI_ANY_SOURCE)) 
+	&& ((msg->tag == tag) || (msg->tag == p_info->__MPI_ANY_TAG) || (tag == p_info->__MPI_ANY_TAG))) {
+
+#if DEBUG_MSG_MATCHING
+      printf("\t\tsrc/dest/tag match\n");
+#endif
+      if((p_comm->status & mask_ok) && (!(p_comm->status & mask_not_ok))) {
+
+#if DEBUG_MSG_MATCHING
+	printf("\t\tOK ! return comm %p\n", p_comm);
+#endif
+	return p_comm;
+      }
+#if DEBUG_MSG_MATCHING
+      else {
+	printf("\t\tbut status does not match (mask_ok = %x, mask_not_ok=%x)\n",
+	       (p_comm->status & mask_ok), (p_comm->status & mask_not_ok) );
+      }
+#endif
     }
   }
+#if DEBUG_MSG_MATCHING
+  printf("\t-> Not found\n");
+#endif
   /* message not found */
   return NULL;
 }
@@ -395,7 +418,7 @@ static void __remove_message_from_pending_comm(struct mpi_p2p_msg_t* msg,
     int i;
     /* check wether the message is complete */
     for (i = 0; i < P2P_NB_TIMES; i++) {
-      if (!msg->times[i])
+      if (!IS_TIME_SET(msg->times[i]))
 	return;
     }
 
@@ -436,7 +459,6 @@ static void __remove_message_from_pending_comm(struct mpi_p2p_msg_t* msg,
 			      thread_id, mpi_req);
       pending_comm = __create_new_pending_comm(dest, msg, comm_type_incoming);
       pending_comm->status |= IRECV_DONE;
-
     } else {
       /* isend found, update its status */
       pending_comm->status |= IRECV_DONE;
@@ -445,9 +467,12 @@ static void __remove_message_from_pending_comm(struct mpi_p2p_msg_t* msg,
       /* create a recv pending comm*/
       new_comm = __create_new_pending_comm(dest, msg, comm_type_incoming);
       new_comm->status |= pending_comm->status;
-
     }
 
+    assert (pending_comm->status & IRECV_DONE);
+    assert (!(pending_comm->status & RWAIT_DONE));
+
+
     assert(msg);
 
     __record_event(start_time, GET_PROCESS_CONTAINER(dest),
@@ -482,23 +507,35 @@ static void __remove_message_from_pending_comm(struct mpi_p2p_msg_t* msg,
       return NULL;
     }
 
+    assert (pending_comm->status & IRECV_DONE);
+
     /* update the status of the pending incoming comm */
     pending_comm->status |= RWAIT_DONE;
     msg = pending_comm->msg;
 
     /* update the outgoing pending comm status */
-    pending_comm_outgoing = __mpi_p2p_find_pending_isend(src, dest, len, tag,
+
+    /* we use msg->src and msg->tag instead of src (resp. tag) because
+     * src may be set to MPI_ANY_SOURCE (resp. MPI_ANY_TAG), while
+     * msg->src (resp msg->tag) contains the actual src (resp. tag)
+     * that was matched (in pending_comm).
+     */
+    pending_comm_outgoing = __mpi_p2p_find_pending_isend(msg->src, dest, len, msg->tag,
 							 NONE_DONE, RWAIT_DONE);
-    if (pending_comm_outgoing)
+    if (pending_comm_outgoing) {
       pending_comm_outgoing->status |= RWAIT_DONE;
+      assert(pending_comm_outgoing->status & IRECV_DONE);
+    }
 
     assert(msg);
 
-	__record_event(stop_time, GET_PROCESS_CONTAINER(dest),
+    __record_event(stop_time, GET_PROCESS_CONTAINER(dest),
 		   EZTRACE_MPI_P2P_RWAIT_ID, msg);
 
-    if (!msg->times[start_rwait])
+    if (!IS_TIME_SET(msg->times[start_rwait])) {
       msg->times[start_rwait] = stop_time;
+    }
+
     msg->times[stop_recv] = stop_time;
 
     if (mpi_req) {
@@ -613,8 +650,9 @@ static void __remove_message_from_pending_comm(struct mpi_p2p_msg_t* msg,
     if (msg->tag == p_info->__MPI_ANY_TAG)
       msg->tag = tag;
 
-    if (!msg->times[start_swait])
+    if (!IS_TIME_SET(msg->times[start_swait])) {
       msg->times[start_swait] = stop_time;
+    }
     msg->times[stop_send] = stop_time;
 
     if (mpi_req) {
