@@ -4,7 +4,6 @@
  * See COPYING in top-level directory.
  */
 
-#define _GNU_SOURCE 1
 #define _REENTRANT
 
 #include <unistd.h>
@@ -29,7 +28,7 @@ static void MPI_Igatherv_prolog(CONST void *sendbuf __attribute__((unused)),
 				CONST int *recvcnts __attribute__((unused)),
 				CONST int *displs __attribute__((unused)),
 				MPI_Datatype recvtype,
-				int root __attribute__((unused)),
+				int root,
 				MPI_Comm comm,
 				MPI_Request *r)
 {
@@ -41,7 +40,7 @@ static void MPI_Igatherv_prolog(CONST void *sendbuf __attribute__((unused)),
   int ssize;
   MPI_Type_size(sendtype, &ssize);
   int data_size = ssize * sendcnt;
-  EZTRACE_EVENT_PACKED_5(EZTRACE_MPI_IGATHERV, (app_ptr)comm, size, rank, (app_ptr)r, data_size);
+  EZTRACE_EVENT_PACKED_6(EZTRACE_MPI_IGATHERV, (app_ptr)comm, size, rank, (app_ptr)r, data_size, root);
 }
 
 static int MPI_Igatherv_core(CONST void *sendbuf, int sendcnt, MPI_Datatype sendtype,
