@@ -1,10 +1,9 @@
 /* -*- c-file-style: "GNU" -*- */
 /*
- * Copyright © CNRS, INRIA, Université Bordeaux 1
+ * Copyright (C) CNRS, INRIA, Universite Bordeaux 1, Telecom SudParis
  * See COPYING in top-level directory.
  */
 
-#define _GNU_SOURCE 1
 #define _REENTRANT
 
 #include <unistd.h>
@@ -33,16 +32,17 @@ static void MPI_Scan_prolog (CONST void *sendbuf __attribute__((unused)),
   libMPI_Comm_size(comm, &size);
   libMPI_Comm_rank(comm, &rank);
 
-  int len;
-  MPI_Type_size(datatype, &len);
+  int ssize;
+  MPI_Type_size(datatype, &ssize);
+  int data_size = ssize * count;
 
-  EZTRACE_EVENT3(FUT_MPI_START_Scan, comm, size, rank);
+  EZTRACE_EVENT_PACKED_4(EZTRACE_MPI_START_Scan, (app_ptr)comm, size, rank, data_size);
 }
 
-static int MPI_Scan_core (CONST void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype,
-			  MPI_Op op, MPI_Comm comm)
+static int MPI_Scan_core(CONST void *sendbuf, void *recvbuf, int count,
+                         MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
 {
-  return libMPI_Scan (sendbuf, recvbuf, count, datatype,op, comm);
+  return libMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
 }
 
 static void MPI_Scan_epilog (CONST void *sendbuf __attribute__((unused)),
@@ -60,25 +60,23 @@ static void MPI_Scan_epilog (CONST void *sendbuf __attribute__((unused)),
   int len;
   MPI_Type_size(datatype, &len);
 
-  EZTRACE_EVENT3(FUT_MPI_STOP_Scan, comm, size, rank);
+    EZTRACE_EVENT_PACKED_3(EZTRACE_MPI_STOP_Scan, (app_ptr)comm, size, rank);
 }
 
-
-int MPI_Scan (CONST void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype,
-	      MPI_Op op, MPI_Comm comm)
+int MPI_Scan(CONST void *sendbuf, void *recvbuf, int count,
+             MPI_Datatype datatype, MPI_Op op, MPI_Comm comm)
 {
   FUNCTION_ENTRY;
 
-  MPI_Scan_prolog (sendbuf, recvbuf, count, datatype,op, comm);
-  int ret = MPI_Scan_core (sendbuf, recvbuf, count, datatype,op, comm);
-  MPI_Scan_epilog (sendbuf, recvbuf, count, datatype,op, comm);
+  MPI_Scan_prolog(sendbuf, recvbuf, count, datatype, op, comm);
+  int ret = MPI_Scan_core(sendbuf, recvbuf, count, datatype, op, comm);
+  MPI_Scan_epilog(sendbuf, recvbuf, count, datatype, op, comm);
   return ret;
 }
 
-
-void mpif_scan_(void *sbuf, void *rbuf, int *count,
-		MPI_Fint *d, MPI_Fint *op, MPI_Fint *c, int *error)
-{
+void mpif_scan_(void *sbuf, void *rbuf, int *count, MPI_Fint *d, MPI_Fint *op,
+                MPI_Fint *c, int *error) {
+  FUNCTION_ENTRY;
   MPI_Datatype c_type = MPI_Type_f2c(*d);
   MPI_Op c_op = MPI_Op_f2c(*op);
   MPI_Comm c_comm = MPI_Comm_f2c(*c);
