@@ -1,6 +1,6 @@
 /* -*- c-file-style: "GNU" -*- */
 /*
- * Copyright  CNRS, INRIA, Universit Bordeaux 1
+ * Copyright (C) CNRS, INRIA, Universite Bordeaux 1, Telecom SudParis
  * See COPYING in top-level directory.
  */
 
@@ -10,14 +10,13 @@
 #include <papi.h>
 #include "ev_codes.h"
 
+#define EZTRACE_PAPI_EVENTS_ID SYSTEM_MODULE_ID(0x06)
+#define EZTRACE_PAPI_PREFIX    GENERATE_SYSTEM_MODULE_PREFIX(EZTRACE_PAPI_EVENTS_ID)
 
-#define PAPI_EVENTS_ID SYSTEM_MODULE_ID(0x06)
-#define PAPI_PREFIX    GENERATE_SYSTEM_MODULE_PREFIX(PAPI_EVENTS_ID)
-
-#define FUT_PAPI_INIT            (PAPI_PREFIX | 0X0001)
-#define FUT_PAPI_INIT_COUNTERS   (PAPI_PREFIX | 0X0002)
-#define FUT_PAPI_MEASUREMENT     (PAPI_PREFIX | 0X0010)
-
+#define EZTRACE_PAPI_INIT                (EZTRACE_PAPI_PREFIX | 0X0001)
+#define EZTRACE_PAPI_INIT_COUNTERS       (EZTRACE_PAPI_PREFIX | 0X0002)
+#define EZTRACE_PAPI_INIT_COUNTER_NAME   (EZTRACE_PAPI_PREFIX | 0X0003)
+#define EZTRACE_PAPI_MEASUREMENT         (EZTRACE_PAPI_PREFIX | 0X0010)
 
 struct papi_counter_id {
   int code;
@@ -30,16 +29,16 @@ static int nb_counters = 0;
 
 static inline void __papi_print_counter_ids() {
   int i;
-  for(i=0; i<nb_counters; i++) {
+  for (i = 0; i < nb_counters; i++) {
     printf("[%d] code %x : %s (%s)\n", i, papi_counter_ids[i].code,
-	   papi_counter_ids[i].code_str, papi_counter_ids[i].description);
+           papi_counter_ids[i].code_str, papi_counter_ids[i].description);
   }
 }
 
 static inline void __papi_print_available_counters() {
   PAPI_event_info_t info;
   int retval;
-  int mask = PAPI_PRESET_ENUM_AVAIL;
+  int mask = PAPI_PRESET_ENUM_AVAIL | PAPI_NTV_ENUM_UMASKS;
   int start = 0 | PAPI_PRESET_MASK;
   int i = start;
 
@@ -48,7 +47,7 @@ static inline void __papi_print_available_counters() {
     if (retval == PAPI_OK) {
       printf("%-20s %s\n", info.symbol, info.long_descr);
     }
-  }  while ((retval = PAPI_enum_event(&i, mask  )) == PAPI_OK);
+  } while ((retval = PAPI_enum_event(&i, mask)) == PAPI_OK);
 }
 
 /* search for a counter name and return the corresponding code.
@@ -56,8 +55,8 @@ static inline void __papi_print_available_counters() {
  */
 static inline int __papi_get_counter_id(char* counter_name) {
   int i;
-  for(i=0; i<nb_counters; i++) {
-    if(strcmp(counter_name, papi_counter_ids[i].code_str) == 0){
+  for (i = 0; i < nb_counters; i++) {
+    if (strcmp(counter_name, papi_counter_ids[i].code_str) == 0) {
       return i;
     }
   }
@@ -66,8 +65,8 @@ static inline int __papi_get_counter_id(char* counter_name) {
 
 static inline int __papi_get_counter_id_by_code(int counter_code) {
   int i;
-  for(i=0; i<nb_counters; i++) {
-    if(counter_code == papi_counter_ids[i].code)
+  for (i = 0; i < nb_counters; i++) {
+    if (counter_code == papi_counter_ids[i].code)
       return i;
   }
   return -1;
@@ -93,18 +92,19 @@ static inline void __papi_init_counter_ids() {
 
   do {
     retval = PAPI_get_event_info(i, &info);
-    if (retval == PAPI_OK) nb_counters ++;
-  }  while ((retval = PAPI_enum_event(&i, mask  )) == PAPI_OK);
-  papi_counter_ids = malloc(sizeof(struct papi_counter_id) *  nb_counters);
+    if (retval == PAPI_OK)
+      nb_counters++;
+  } while ((retval = PAPI_enum_event(&i, mask)) == PAPI_OK);
+  papi_counter_ids = malloc(sizeof(struct papi_counter_id) * nb_counters);
   i = start;
 
   do {
     retval = PAPI_get_event_info(i, &info);
     if (retval == PAPI_OK) {
       define_counter(cur_counter, info.event_code, info.symbol, info.long_descr);
-      cur_counter ++;
+      cur_counter++;
     }
-  }  while ((retval = PAPI_enum_event(&i, mask  )) == PAPI_OK);
+  } while ((retval = PAPI_enum_event(&i, mask)) == PAPI_OK);
 }
 
 #endif	/* _PAPI_EV_CODES_H__ */
