@@ -8,13 +8,6 @@
 #include <inttypes.h>
 
 static int recording_stats = 0;
-static double duration_critical_section = 0.0; // total duration of critical sections
-/* todo:
- * - this should be computed for each process
- * - for now, this records the cumulated duration of critical section. If a thread calls
- *   mutex_lock(l1); mutex_lock(l2); the recorded duration of the critical section is twice
- *   the time spent holding l1.
- */
 
 #define PTHREAD_CHANGE() if(!recording_stats) CHANGE()
 
@@ -25,7 +18,13 @@ struct pthread_process_info_t {
 };
 
 enum lock_type_t {
-  MUTEX, SPINLOCK, RWLOCK, SEMAPHORE, BARRIER, CONDITION
+  MUTEX=0,
+  SPINLOCK=1,
+  RWLOCK=2,
+  SEMAPHORE=3,
+  BARRIER=4,
+  CONDITION=5,
+  NB_LOCK_TYPES=6
 };
 
 /* structure that contains information on a lock(mutex, spinloc, etc.)
@@ -37,48 +36,66 @@ struct pthread_lock_info_t {
   char* info; /* name of the lock */
   /* statistics: */
   uint32_t nb_acquire; /* number of lock_stop */
-  uint64_t last_owner_tid; /* tid of the last thread that acquired the lock */
+  int last_owner_tid; /* tid of the last thread that acquired the lock */
   float time_acquire; /* last time the lock was acquired */
   float time_release; /* last time the lock was released */
+
   double duration_critical_section; /* time spent in critical section */
+  double duration_acquire; /* time spent in waiting for a lock */
+
   /* todo: improve statistics:
    * - compute min/max/average critical section duration
    * - record the time spent waiting for the lock
    */
 };
 
+struct pthread_thread_info_t {
+  float start_acquire[NB_LOCK_TYPES]; /* date of the last start_lock event */
+  float duration_acquire[NB_LOCK_TYPES]; /* time spent in waiting for a lock */
+  float duration_critical_section[NB_LOCK_TYPES]; /* time spent in a critical section */
+  struct thread_info_t *p_thread;
+};
+
+static void __lock_type_to_str(char *str, enum lock_type_t l) {
+  switch (l) {
+  case MUTEX:
+    strcpy(str, "mutex");
+    break;
+  case SPINLOCK:
+    strcpy(str, "spinlock");
+    break;
+  case RWLOCK:
+    strcpy(str, "rwlock");
+    break;
+  case SEMAPHORE:
+    strcpy(str, "semaphore");
+    break;
+  case BARRIER:
+    strcpy(str, "barrier");
+    break;
+  case CONDITION:
+    strcpy(str, "condition");
+    break;
+  default:
+    strcpy(str, "<<unknown>>");
+  };
+}
+
 /* print a message that warns about a dangerous behavior. Thread 1 locked, but thread2 unlocked.
  * return 1 if this behavior is detected or 0 otherwise
  */
 static int __check_lock_mismatch(struct pthread_lock_info_t *p_info,
-                                 struct process_info_t *p_process,
-                                 uint64_t cur_tid) {
+                                 struct pthread_thread_info_t *p_thread) {
+  int cur_tid = p_thread->p_thread->tid;
+
   if (p_info->last_owner_tid == cur_tid)
     return 0;
 
-  printf("Warning: t=%f - %s\tthread %"PRIu64" locked ", CURRENT,
-      p_process->container->id, p_info->last_owner_tid);
-  switch (p_info->lock) {
-  case MUTEX:
-    printf("mutex");
-    break;
-  case SPINLOCK:
-    printf("spinlock");
-    break;
-  case RWLOCK:
-    printf("rwlock");
-    break;
-  case SEMAPHORE:
-    printf("semaphore");
-    break;
-  case BARRIER:
-    printf("barrier");
-    break;
-  case CONDITION:
-    printf("condition");
-    break;
-  };
-  printf(" %p, but thread %"PRIu64" unlocks it!\n", (void*)p_info->ptr, cur_tid);
+  char lock_type_str [80];
+  __lock_type_to_str(lock_type_str, p_info->lock);
+
+  printf("Warning: t=%f - thread %d locked ", CURRENT, p_info->last_owner_tid);
+  printf("%s %p, but thread %d unlocks it!\n", lock_type_str, (void*)p_info->ptr, cur_tid);
   return 1;
 }
 
@@ -123,25 +140,39 @@ static struct pthread_process_info_t *__register_process_hook(
 /* add the current duration of critical section (time_release-time_acquire) to the
  * sum of critical sections
  */
-#define update_duration_critical_section(p_lock)			\
+#define update_duration_critical_section(p_lock, p_info)		\
   do {									\
     p_lock->duration_critical_section += p_lock->time_release - p_lock->time_acquire; \
-    duration_critical_section += p_lock->time_release - p_lock->time_acquire; \
+    p_info->duration_critical_section[p_lock->lock] += p_lock->time_release - p_lock->time_acquire; \
+  } while(0)
+
+#define update_duration_acquire(p_lock, p_info)				\
+  do{									\
+    p_lock->duration_acquire += p_lock->time_acquire - p_info->start_acquire[p_lock->lock]; \
+    p_info->duration_acquire[p_lock->lock] += p_lock->time_acquire - p_info->start_acquire[p_lock->lock]; \
+  } while(0)
+
+/* record informations on a lock start acquire event (for statistics) */
+#define record_lock_start_acquire(p_lock, p_info)	\
+  do {							\
+    p_info->start_acquire[p_lock->lock] = CURRENT;	\
   } while(0)
 
 /* record informations on a lock acquire event (for statistics) */
-#define record_lock_acquired(p_lock)		\
-  do {						\
-    p_lock->time_acquire = CURRENT;		\
-    p_lock->nb_acquire++;			\
-    p_lock->last_owner_tid = CUR_THREAD_ID;	\
+#define record_lock_acquired(p_lock, p_info)		\
+  do {							\
+    p_lock->time_acquire = CURRENT;			\
+    update_duration_acquire(p_lock, p_info);		\
+    p_lock->nb_acquire++;				\
+    p_lock->last_owner_tid = p_info->p_thread->tid;	\
   } while(0)
 
 /* record informations on a lock release event (for statistics) */
-#define record_lock_release(p_lock)		\
-  do {						\
-    p_lock->time_release = CURRENT;		\
-    update_duration_critical_section(p_lock);	\
+#define record_lock_release(p_lock, p_info)		\
+  do {							\
+    p_lock->time_release = CURRENT;			\
+    update_duration_critical_section(p_lock, p_info);	\
+    __check_lock_mismatch(p_lock, p_info);		\
   }while(0)
 
 /* Free the lock_info_list attached to a process
@@ -205,9 +236,8 @@ __new_lock_info(struct pthread_process_info_t* p_info, app_ptr ptr,
   /* initialize statistics */
   res->nb_acquire = 0;
   res->last_owner_tid = -1;
-  res->time_acquire = 0;
-  res->time_release = 0;
   res->duration_critical_section = 0;
+  res->duration_acquire = 0;
 
   res->token.data = res;
 
@@ -232,4 +262,3 @@ static struct pthread_lock_info_t*__find_lock_info(struct ezt_list_t *lock_list,
   }
   return NULL;
 }
-
