@@ -1,4 +1,9 @@
+/* -*- c-file-style: "GNU" -*- */
 /*
+ * Copyright (C) CNRS, INRIA, Université Bordeaux 1, Télécom SudParis
+ * See COPYING in top-level directory.
+ *
+ *
  * memory.h -- handles memory allocation in the target process
  *
  * Provided methods:
@@ -17,8 +22,10 @@
 #include <stdint.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include "types.h"
 
-uint64_t allocate_buffer(pid_t child, size_t size);
-void correct_buffer_allocation(pid_t child, size_t required_size, size_t actual_size);
+word_uint allocate_buffer(pid_t child, size_t size);
+void correct_buffer_allocation(pid_t child, size_t required_size,
+                               size_t actual_size);
 
 #endif /* PPTRACE_MEMORY_H_ */
